use std::sync::Arc;

use axum::{Extension, Json};
use foxchat::{
    http::ApiError,
    model::{http::guild::CreateGuildParams, Guild, user::PartialUser},
    FoxError,
};
use ulid::Ulid;

use crate::{app_state::AppState, fed::FoxRequestData, model::user::User};

pub async fn create_guild(
    Extension(state): Extension<Arc<AppState>>,
    request: FoxRequestData,
    Json(params): Json<CreateGuildParams>,
) -> Result<Json<Guild>, ApiError> {
    let user_id = request.user_id.ok_or(FoxError::MissingUser)?;

    let user = User::get(&state, &request.instance, user_id).await?;

    let guild = sqlx::query!(
        "insert into guilds (id, owner_id, name) values ($1, $2, $3) returning *",
        Ulid::new().to_string(),
        user.id,
        params.name
    )
    .fetch_one(&state.pool)
    .await?;

    Ok(Json(Guild {
        id: guild.id,
        name: guild.name,
        owner: PartialUser {
            id: user.id,
            username: user.username,
            instance: request.instance.domain,
        }
    }))
}
