use std::sync::Arc;

use eyre::Result;
use foxchat::{fed, model::User as HttpUser};
use ulid::Ulid;

use crate::app_state::AppState;

use super::identity_instance::IdentityInstance;

pub struct User {
    pub id: String,
    pub instance_id: String,
    pub remote_user_id: String,
    pub username: String,
    pub avatar: Option<String>,
}

impl User {
    pub async fn get(
        state: &Arc<AppState>,
        instance: &IdentityInstance,
        remote_id: String,
    ) -> Result<User> {
        if let Some(user) = sqlx::query_as!(
            User,
            "select * from users where instance_id = $1 and remote_user_id = $2",
            instance.id,
            remote_id
        )
        .fetch_optional(&state.pool)
        .await?
        {
            return Ok(user);
        }

        let http_user = fed::get::<HttpUser>(
            &state.private_key,
            &state.config.domain,
            &instance.domain,
            &format!("/_fox/ident/users/{}", remote_id),
            None,
        )
        .await?;

        let user = sqlx::query_as!(
            User,
            "insert into users (id, instance_id, remote_user_id, username, avatar) values ($1, $2, $3, $4, $5) returning *",
            Ulid::new().to_string(),
            instance.id,
            http_user.id,
            http_user.username,
            http_user.avatar_url
        )
        .fetch_one(&state.pool)
        .await?;

        Ok(user)
    }
}
